package j3d.cr325;

import com.sun.j3d.loaders.Scene;
import com.sun.j3d.loaders.objectfile.ObjectFile;
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.geometry.Sphere;
import j3d.Utils;

import javax.media.j3d.*;
import javax.vecmath.Color3f;
import javax.vecmath.Point3d;
import java.applet.Applet;
import java.awt.*;
import java.io.FileNotFoundException;

public class SimpleGame extends Applet {

    private BoundingSphere bsBounds
            = new BoundingSphere(new Point3d(0.0, 0.0, 0.0), 100.0);
    /**
     * Alpha used to drive the duck animation
     */
    Alpha targetAlpha = new Alpha(-1, 0, 0, 3000, 0, 0);
    /**
     * Used to drive the ball animation
     */
    Alpha bulletAlpha = new Alpha(1, 0, 0, 500, 0, 0);
    /**
     * Used to move the ball
     */
    PositionInterpolator bulletInterpolator = null;
    /**
     * Used to rotate the gun
     */
    TransformGroup gunTransformGroup = Shapes.getReadWriteTransformGroup();

    private Scene getDuckObject(String dataDir)
            throws FileNotFoundException {
        return new ObjectFile().load(dataDir + "duck.obj");
    }

    private Scene getDeadDuckObject(String dataDir)
            throws FileNotFoundException {
        return new ObjectFile().load(dataDir + "deadduck.obj");
    }


    /**
     * This builds the ball that acts as the bullet for our gun.
     * The ball is created from a sphere primitive, and a transform
     * group and interpolator are added so that we can 'fire' the
     * bullet.
     *
     * @return BranchGroup that is the root of the ball branch.
     */
    protected BranchGroup getBulletGroup() {
        BranchGroup bullet = new BranchGroup();

        Appearance bulletAppearance = new Appearance();
        Color3f ambientColour = new Color3f(1.0f, 0.0f, 0.0f);
        Color3f emissiveColour = new Color3f(0.0f, 0.0f, 0.0f);
        Color3f specularColour = new Color3f(1.0f, 1.0f, 1.0f);
        Color3f diffuseColour = new Color3f(1.0f, 0.0f, 0.0f);
        float shininess = 20.0f;
        bulletAppearance.setMaterial(new Material(ambientColour, emissiveColour,
                diffuseColour, specularColour, shininess));

        Sphere ball = new Sphere(0.2f, bulletAppearance);

        TransformGroup ballMovXfmGrp = new TransformGroup();
        ballMovXfmGrp.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
        ballMovXfmGrp.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
        ballMovXfmGrp.addChild(ball);
        bullet.addChild(ballMovXfmGrp);


        bulletInterpolator = new PositionInterpolator(bulletAlpha,
                ballMovXfmGrp,
                new Transform3D(), 0.0f, 50.0f);
        bulletInterpolator.setSchedulingBounds(getBsBounds());
        bullet.addChild(bulletInterpolator);

        return bullet;

    }

    protected BranchGroup getContentBranch() {
        BranchGroup bg = new BranchGroup();
        bg.addChild(Shapes.getTreeBranchGroup());
        bg.addChild(Shapes.getHousesBranchGroup());
        bg.addChild(Shapes.getBackgroundBranchGroup());
        Node theTarget = Target.getTargetGroup();
        bg.addChild(theTarget);
        Node theBulletNode = getBulletGroup();
        bg.addChild(theBulletNode);
        bg.addChild(new TargetBehavior(theTarget,
                Target.getTargetSwitch(),
                targetAlpha,
                getBsBounds()));
        bg.addChild(new GunBehavior(bulletAlpha,
                bulletInterpolator,
                gunTransformGroup,
                getBsBounds()));
        bg.addChild(Shapes.getGunBranchGroup(gunTransformGroup));
        Utils.addLights(getBsBounds(), bg);
        return bg;
    }


    private SimpleGame() {
        Canvas3D c3d
                = Utils.getCanvas3D(this);
        Locale l
                = new Locale(new VirtualUniverse());
        l.addBranchGraph(Shapes.buildViewBranch(c3d));
        l.addBranchGraph(getContentBranch());
        setLayout(new BorderLayout());
        add("Center", c3d);
    }

    public static void main(String[] args) {
        new MainFrame(createSimpleGame(), 400, 400);
    }

    public static SimpleGame createSimpleGame() {
        return new SimpleGame();
    }

    public BoundingSphere getBsBounds() {
        return bsBounds;
    }

}
